#
# spec file for package tcmu-runner
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%if 0%{?sle_version} == 0
%define         build_handler_glusterfs 1
%endif

%if 0%{?sle_version} == 0 || 0%{?sle_version} >= 120300
%ifarch aarch64 x86_64
%define         build_handler_rbd 1
%endif
%endif

Name:           tcmu-runner
Version:        1.3.0
Release:        lp150.1.1
Summary:        A userspace daemon that handles the LIO TCM-User backstore
License:        Apache-2.0
Group:          System/Management
Url:            https://github.com/agrover/%{name}
Source:         %{name}-%{version}.tar.gz
Patch1:         %{name}-handler_file-add-libtcmu.patch
Patch7:         %{name}-remove-handler-path-install-prefix.patch
Patch9:         %{name}-remove-uio-include-for-iovec.patch
Patch10:        api-avoid-compiler-Werror-return-type-failures.patch
Patch11:        build-drop-versionless-libtcmu.so-symlink.patch
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
BuildRequires:  cmake
BuildRequires:  glib2-devel
BuildRequires:  glibc-devel
%if 0%{?build_handler_glusterfs}
BuildRequires:  glusterfs-devel
%endif
%if 0%{?build_handler_rbd}
BuildRequires:  librbd-devel
%endif
BuildRequires:  libkmod-devel
BuildRequires:  libnl3-devel
BuildRequires:  systemd-rpm-macros
BuildRequires:  zlib-devel
%{?systemd_requires}

%description
LIO is the SCSI target in the Linux kernel. It is entirely kernel
code, and allows exported SCSI logical  units (LUNs) to be backed
by regular files or block devices. But, if we want to get fancier with
the capabilities of the device we're emulating, the kernel is not
necessarily the right place. While there are userspace libraries for
compression, encryption, and clustered storage solutions like
Ceph or Gluster, these are not accessible from the kernel. 

The TCMU userspace-passthrough backstore allows a userspace process to
handle requests to a LUN. But since the kernel-user interface that
TCMU provides must be fast and flexible, it is complex enough that
we'd like to avoid each  userspace handler having to write boilerplate
code.

tcmu-runner handles the messy details of the TCMU interface -- UIO,
netlink, pthreads, and DBus -- and exports a more friendly C plugin
module API. Modules using this API are called "TCMU handlers". Handler
authors can write code just to handle the SCSI commands as desired,
and can also link with whatever userspace libraries they like.

%package -n libtcmu2
Summary:        Runtime libraries for tcmu-runner
Group:          System/Libraries

%description -n libtcmu2
This package contains the runtime libraries for tcmu-runner.

%if 0%{?build_handler_glusterfs}
%package handler-glusterfs
Summary:        GlusterFS handler for tcmu-runner
Group:          System/Libraries
Requires:       tcmu-runner = %{version}

%description handler-glusterfs
This package contains the GlusterFS handler for tcmu-runner, which
allows for LIO/tcmu logical units to be backed by GlusterFS provisioned
storage.
%endif

%if 0%{?build_handler_rbd}
%package handler-rbd
Summary:        Ceph RBD handler for tcmu-runner
Group:          System/Libraries
Requires:       tcmu-runner = %{version}

%description handler-rbd
This package contains the Ceph RADOS Block Device (RBD) handler for
tcmu-runner, which allows for LIO/tcmu logical units to be backed by
RBD images.
%endif

%prep
%setup
%patch1 -p1
%patch7 -p1
%if 0%{?suse_version} >= 1330
%patch9 -p1
%endif
%patch10 -p1
%patch11 -p1

%build
CMAKE_OPTIONS="\
	-DSUPPORT_SYSTEMD=1 \
%if 0%{?build_handler_glusterfs}
	-Dwith-glfs=1 \
%else
	-Dwith-glfs=0 \
%endif
%if 0%{?build_handler_rbd}
	-Dwith-rbd=1 \
%else
	-Dwith-rbd=0 \
%endif
"
%cmake -DCMAKE_SHARED_LINKER_FLAGS="-Wl,--as-needed -Wl,-z,now" \
	${CMAKE_OPTIONS}
make %{?_smp_mflags}

%install
%cmake_install
install -d -m 755 %{buildroot}/%{_sbindir}
ln -s %{_sbindir}/service %{buildroot}/%{_sbindir}/rctcmu-runner

%post
%{run_ldconfig}
%{service_add_post tcmu-runner.service}

%postun
%{service_del_postun tcmu-runner.service}
%{run_ldconfig}

%pre
%{service_add_pre tcmu-runner.service}

%preun
%{service_del_preun tcmu-runner.service}

%post -n libtcmu2 -p /sbin/ldconfig

%postun -n libtcmu2 -p /sbin/ldconfig

%files
%defattr(-,root,root)
%{_bindir}/tcmu-runner
%{_sbindir}/rctcmu-runner
%doc README.md LICENSE
%dir %{_libdir}/tcmu-runner
%{_libdir}/tcmu-runner/handler_qcow.so
%dir %{_datadir}/dbus-1
%dir %{_datadir}/dbus-1/system-services
%{_datadir}/dbus-1/system-services/org.kernel.TCMUService1.service
%dir %{_sysconfdir}/dbus-1
%dir %{_sysconfdir}/dbus-1/system.d
%config %{_sysconfdir}/dbus-1/system.d/tcmu-runner.conf
%{_unitdir}/tcmu-runner.service
%doc %{_mandir}/man8/tcmu-runner.8%{ext_man}

%files -n libtcmu2
%defattr(-,root,root)
%{_libdir}/libtcmu.so.2
%dir %{_sysconfdir}/tcmu
%config(noreplace) %{_sysconfdir}/tcmu/tcmu.conf

%if 0%{?build_handler_glusterfs}
%files handler-glusterfs
%{_libdir}/tcmu-runner/handler_glfs.so
%endif

%if 0%{?build_handler_rbd}
%files handler-rbd
%{_libdir}/tcmu-runner/handler_rbd.so
%endif

%changelog
* Fri Feb  9 2018 ddiss@suse.com
- Update to version 1.3.0; (fate#322524)
  + Improved Ceph RBD integration
  + Support XCOPY copy-offload commands
  + Support COMPARE_AND_WRITE, with Ceph OSD request mapping
  + Asynchronous I/O improvements and bug fixes
- Remove a number of downstream patches now carried upstream:
  + glfs-discard-glfs_check_config
  + fixed-local-dos-when-unregisterhandler-was-called-for-a-not-existing-handler
  + only-allow-dynamic-unregisterhandler-for-external-handlers-thereby-fixing-dos
  + fixed-a-number-of-memory-leaks-with-deregistering-of-dbus-handlers
  + removed-all-check_config-callback-implementations-to-avoid-security-issues
  + tcmu-runner-add-local-target_core_user-h
- Remove development packages (bsc#1080619)
  + API discontinued upstream
- Add fix for -Werror-return-type failures:
  + api-avoid-compiler-Werror-return-type-failures.patch
- Add fix for library versioning:
  + build-drop-versionless-libtcmu.so-symlink.patch
* Wed Sep 20 2017 lduncan@suse.com
- Add two patches: make target_core_user.h local, to insulate
  tcmu-runner from kernel version/data structure/naming
  changes, and add include of <sys/uio.h> to that file,
  to fix "stuct iovec" not defined issue  (bsc#1058303),
  adding patches:
  * tcmu-runner-add-local-target_core_user-h
  * tcmu-runner-remove-uio-include-for-iovec.patch
  The latter patch is added only for SUSE version >= 13.3,
  since it is only needed there.
* Wed Aug  9 2017 lduncan@suse.com
- Remove install prefix from beginning of install libdir when
  setting tcmu-runner plugin path (bsc#1052795), adding
  SUSE-specific patch:
  * tcmu-runner-remove-handler-path-install-prefix.patch
* Thu Jul 20 2017 lduncan@suse.com
- One more update from Security Audit, adding one more patch:
  - removed-all-check_config-callback-implementations-to-avoid-security-issues
    (bsc#1049491)
* Thu Jul 20 2017 lduncan@suse.com
- Updates based on Security Audit, adding 4 patches:
  * glfs-discard-glfs_check_config (bsc#1049485)
  * fixed-local-dos-when-unregisterhandler-was-called-for-a-not-existing-handler
    (bsc#1049488)
  * only-allow-dynamic-unregisterhandler-for-external-handlers-thereby-fixing-dos
    (bsc#1049489)
  * fixed-a-number-of-memory-leaks-with-deregistering-of-dbus-handlers
    (bsc#1049490)
* Mon May 15 2017 ddiss@suse.com
- Make Ceph RBD handler aarch64 and x86-64 only for now
  + librbd isn't yet available for all platforms
* Thu May 11 2017 ddiss@suse.com
- Build the Ceph RBD handler and package separately; (fate#322524).
- Split the GlusterFS handler into a separate package
  + avoid adding an unnecessary dependencies to tcmu-runner core
* Thu May 11 2017 ddiss@suse.com
- Explicitly disable the GlusterFS handler for SLE builds.
* Thu May 11 2017 ddiss@suse.com
- Update to upstream version 1.2.0; (bsc#1038712).
  + Add remove devices when tcmulib teardown
  + fix builds done outside of source directory
  + free tcmu_device after dev->fd is closed
  + fix Read_Capacity_16
  + detect INQURY response truncation
  + fix incorrect memcpy length
  + fix inquiry buffer length calculations
  + add tcmu.conf for log level configuration
- Remove upstreamed fixes:
  + tcmu-runner-include-falloc-in-qcow.patch
  + tcmu-runner-let-cmake-install-man-page.patch
  + tcmu-runner-Use-CMake-FULL-library-directory-for-SOs.patch
  + include-tcmu-runner-h-in-install.patch
* Sun Aug 21 2016 jengelh@inai.de
- rename libtcmu1-devel to libtcmu-devel
- drop unnecessary %%oname indirect macro
* Tue Aug 16 2016 lduncan@suse.com
- Updated SPEC file to break library into separate packages
- Install tcmu-runner.h, adding:
  * include-tcmu-runner-h-in-install.patch
* Fri Jul 29 2016 jengelh@inai.de
- Drop silly buildrequires on libgfapi0/libglib-2_0-0.
  Abolish old macros.
* Sat May 28 2016 lduncan@suse.com
- Updated SPEC file to install LICENSE file (bsc#982130)
* Thu May  5 2016 lduncan@suse.com
- Created from upstream version 1.0.4
